@@ -5,137 +5,48 @@
  *
  */
 #include "state.h"
+#include "candidate.h"
+#include "context.h"
+#include "engine.h"
+#include "ime.h"
 #include "pinyinhelper_public.h"
 #include "punctuation_public.h"
 #include "quickphrase_public.h"
 #include <algorithm>
+#include <cstddef>
+#include <cstdint>
+#include <fcitx-utils/capabilityflags.h>
 #include <fcitx-utils/event.h>
+#include <fcitx-utils/i18n.h>
+#include <fcitx-utils/key.h>
 #include <fcitx-utils/keysym.h>
 #include <fcitx-utils/stringutils.h>
 #include <fcitx-utils/utf8.h>
+#include <fcitx/candidatelist.h>
 #include <fcitx/event.h>
 #include <fcitx/inputcontext.h>
+#include <fcitx/inputmethodentry.h>
 #include <fcitx/inputpanel.h>
+#include <fcitx/text.h>
+#include <fcitx/userinterface.h>
 #include <fmt/core.h>
 #include <fmt/format.h>
+#include <initializer_list>
+#include <iterator>
 #include <libime/core/historybigram.h>
 #include <libime/pinyin/pinyinencoder.h>
 #include <libime/pinyin/shuangpinprofile.h>
+#include <libime/table/tablebaseddictionary.h>
+#include <map>
+#include <memory>
+#include <string>
+#include <string_view>
+#include <tuple>
+#include <utility>
+#include <vector>
 
 namespace fcitx {
 
-namespace {
-
-class CommitAfterSelectWrapper {
-public:
-    CommitAfterSelectWrapper(TableState *state) : state_(state) {
-        if (auto *context = state->updateContext(nullptr)) {
-            commitFrom_ = static_cast<int>(context->selectedSize());
-        }
-    }
-
-    ~CommitAfterSelectWrapper() {
-        if (commitFrom_ >= 0) {
-            state_->commitAfterSelect(commitFrom_);
-        }
-    }
-
-private:
-    TableState *state_;
-    int commitFrom_ = -1;
-};
-
-class TableCandidateWord : public CandidateWord {
-public:
-    TableCandidateWord(TableEngine *engine, Text text, size_t idx)
-        : CandidateWord(std::move(text)), engine_(engine), idx_(idx) {}
-
-    void select(InputContext *inputContext) const override {
-        auto *state = inputContext->propertyFor(&engine_->factory());
-        // nullptr means use the last requested entry.
-        auto *context = state->updateContext(nullptr);
-        if (!context || idx_ >= context->candidates().size()) {
-            return;
-        }
-        if (state->mode() == TableMode::ForgetWord) {
-            state->forgetCandidateWord(idx_);
-            return;
-        }
-        {
-            const CommitAfterSelectWrapper commitAfterSelectRAII(state);
-            context->select(idx_);
-        }
-        if (context->selected()) {
-            state->commitBuffer(true);
-        }
-        state->updateUI(/*keepOldCursor=*/false, /*maybePredict=*/true);
-    }
-
-    TableEngine *engine_;
-    size_t idx_;
-};
-
-class TablePinyinCandidateWord : public CandidateWord {
-public:
-    TablePinyinCandidateWord(TableEngine *engine, std::string word,
-                             const libime::TableBasedDictionary &dict,
-                             bool customHint)
-        : engine_(engine), word_(std::move(word)) {
-        Text text;
-        text.append(word_);
-        if (utf8::lengthValidated(word_) == 1) {
-            if (auto code = dict.reverseLookup(word_); !code.empty()) {
-                text.append(" ~ ");
-                if (customHint) {
-                    text.append(dict.hint(code));
-                } else {
-                    text.append(std::move(code));
-                }
-            }
-        }
-        setText(std::move(text));
-    }
-
-    void select(InputContext *inputContext) const override {
-        auto *state = inputContext->propertyFor(&engine_->factory());
-        inputContext->commitString(word_);
-        state->pushLastCommit("", word_);
-        state->resetAndPredict();
-    }
-
-    TableEngine *engine_;
-    std::string word_;
-};
-
-class TablePunctuationCandidateWord : public CandidateWord {
-public:
-    TablePunctuationCandidateWord(TableState *state, std::string word,
-                                  bool isHalf)
-        : state_(state), word_(std::move(word)) {
-        Text text;
-        if (isHalf) {
-            text.append(fmt::format(_("{0} (Half)"), word_));
-        } else {
-            text.append(word_);
-        }
-        setText(std::move(text));
-    }
-
-    void select(InputContext *inputContext) const override {
-        state_->commitBuffer(true);
-        inputContext->commitString(word_);
-        state_->reset();
-    }
-
-    const std::string &word() const { return word_; }
-
-private:
-    TableState *state_;
-    std::string word_;
-};
-
-} // namespace
-
 TableContext *TableState::updateContext(const InputMethodEntry *entry) {
     if (!entry || lastContext_ == entry->uniqueName()) {
         return context_.get();
@@ -260,22 +171,6 @@ void TableState::reset(const InputMethodEntry *entry) {
     engine_->instance()->resetCompose(ic_);
 }
 
-class TablePredictCandidateWord : public CandidateWord {
-public:
-    TablePredictCandidateWord(TableState *state, std::string word)
-        : CandidateWord(Text(word)), state_(state), word_(std::move(word)) {}
-
-    void select(InputContext *inputContext) const override {
-        state_->commitBuffer(true);
-        inputContext->commitString(word_);
-        state_->pushLastCommit("", word_);
-        state_->resetAndPredict();
-    }
-
-    TableState *state_;
-    std::string word_;
-};
-
 std::unique_ptr<CandidateList>
 TableState::predictCandidateList(const std::vector<std::string> &words) {
     if (words.empty()) {
@@ -869,7 +764,8 @@ void TableState::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
     }
 
     if ((mode_ == TableMode::Normal || mode_ == TableMode::Pinyin) &&
-        !event.key().hasModifier()) {
+        !event.key().states().testAny(
+            KeyStates{KeyState::Ctrl, KeyState::Super})) {
         auto compose = engine_->instance()->processComposeString(
             inputContext, event.key().sym());
         if (!compose) {
@@ -1071,7 +967,7 @@ void TableState::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
                 // forward the original key is the best choice.
                 cancelLastEvent_ =
                     engine_->instance()->eventLoop().addTimeEvent(
-                        CLOCK_MONOTONIC, now(CLOCK_MONOTONIC) + 300, 0,
+                        CLOCK_MONOTONIC, now(CLOCK_MONOTONIC) + 3000, 1,
                         [this, ref = inputContext->watch(),
                          puncStr = puncStr](EventSourceTime *, uint64_t) {
                             if (auto *inputContext = ref.get()) {
@@ -1436,6 +1332,8 @@ void TableState::updateUI(bool keepOldCursor, bool maybePredict) {
                 }
                 candidateList->setGlobalCursorIndex(cursor);
             }
+            candidateList->setActionableImpl(
+                std::make_unique<TableActionableCandidateList>(this));
             inputPanel.setCandidateList(std::move(candidateList));
         }
         const bool useClientPreedit =
